@@ -9,7 +9,7 @@ Kafka - Apache Kafka interface for Perl.
 
 =head1 VERSION
 
-This documentation refers to C<Kafka> package version 1.03 .
+This documentation refers to C<Kafka> package version 1.04 .
 
 =cut
 
@@ -21,7 +21,7 @@ use warnings;
 
 
 
-our $VERSION = '1.03';
+our $VERSION = '1.04';
 
 use Exporter qw(
     import
@@ -818,10 +818,10 @@ const our $ERROR_SEND_NO_ACK                    => -1002;
 
 =item C<ERROR_CANNOT_RECV>
 
--1003 - Cannot receive
+-1.04 - Cannot receive
 
 =cut
-const our $ERROR_CANNOT_RECV                    => -1003;
+const our $ERROR_CANNOT_RECV                    => -1.04;
 
 =item C<ERROR_CANNOT_BIND>
 
@@ -1647,6 +1647,8 @@ Vlad Marchenko
 
 Damien Krotkine
 
+Greg Franklin
+
 =head1 COPYRIGHT AND LICENSE
 
 Copyright (C) 2012-2017 by TrackingSoft LLC.
