@@ -6,7 +6,7 @@ Kafka::Producer - Perl interface for Kafka producer client.
 
 =head1 VERSION
 
-This documentation refers to C<Kafka::Producer> version 1.03 .
+This documentation refers to C<Kafka::Producer> version 1.04 .
 
 =cut
 
@@ -18,7 +18,7 @@ use warnings;
 
 
 
-our $VERSION = '1.03';
+our $VERSION = '1.04';
 
 
 
@@ -202,7 +202,7 @@ L<Kafka|Kafka> module.
 
 =cut
 sub new {
-    my ( $class, @args ) = @_;
+    my ( $class, %p ) = @_;
 
     my $self = bless {
         Connection      => undef,
@@ -211,10 +211,7 @@ sub new {
         Timeout         => $REQUEST_TIMEOUT,
     }, $class;
 
-    while ( @args ) {
-        my $k = shift @args;
-        $self->{ $k } = shift @args if exists $self->{ $k };
-    }
+    exists $p{$_} and $self->{$_} = $p{$_} foreach keys %$self;
 
     $self->{ClientId}       //= 'producer';
 
